#include "LayerVariable.h"
#include "Renderable.h"

LayerVariable::LayerVariable(void)
{
	//defaults, so we don't have garbage vars!
	renderable_value = NULL;
	m_overrideValue = 0.0f;
	m_maxValue = 1.0f;
	m_minValue = -1.0f;
	belongs_to_renderable = NULL;
	memset(name, 0, sizeof(name));
}

const char* LayerVariable::GetName()
{
	return name;
}

void LayerVariable::Initialize(Renderable *belongs_to, const char* newName, float overrideValue, float maxVal, float minVal)
{
	belongs_to_renderable = belongs_to;
	m_overrideValue = overrideValue;
	m_maxValue = maxVal;
	m_minValue = minVal;
	sprintf_s(name, newName);
}

LayerVariable::~LayerVariable(void)
{
	RemoveRenderable();
}

void LayerVariable::RemoveRenderable()
{
	if(renderable_value != NULL)
		delete renderable_value;
	renderable_value = NULL;
}

float LayerVariable::GetValue(int x, int y)
{
	if(renderable_value == NULL)
		return GetOverrideValue();
	else
	{
		//GetValueAt returns a value between 0 and 1.
		//Rescale to proper range:
		float scale = renderable_value->GetValueAt(x,y);
		return (((m_maxValue - m_minValue) * scale) + m_minValue);
	}

	return 0;
}

float LayerVariable::GetOverrideValue()
{
	return m_overrideValue;
}

void LayerVariable::SetValue(float value)
{
	//clamp values.
	float finalValue;
	finalValue = MAX(m_minValue, value);
	finalValue = MIN(m_maxValue, finalValue);

	m_overrideValue = finalValue;
}

Renderable* LayerVariable::GetRenderable()
{
	return renderable_value;
}

void LayerVariable::SetRenderable(Renderable* layer)
{
	if(renderable_value != NULL)
		RemoveRenderable();

	renderable_value = layer;
	layer->ParentLayer = belongs_to_renderable;
}

void LayerVariable::MoveCoordinates(int delta_x, int delta_y)
{
	if(renderable_value != NULL)
		renderable_value->MoveCoordinates(delta_x, delta_y);
}

void LayerVariable::IncrementZoom(float deltazoom)
{
	if(renderable_value != NULL)
		renderable_value->IncrementZoom(deltazoom);
}