#include "ProgramManager.h"
#include <utils.h>

#include "NoiseSlice.h"

ProgramManager* ProgramManager::m_Manager = NULL;

ProgramManager::ProgramManager(void)
{
	m_Manager = this;
	m_MouseState = MOUSE_INACTIVE;
	mousedown_x = 0;
	mousedown_y = 0;

	m_window_id = 0;
}

ProgramManager::~ProgramManager(void)
{
	m_Manager = NULL;
}

void ProgramManager::menu_Reset(int value)
{
	switch(value)
	{
	case 1:
		m_Manager->m_layerManager.ResetCoords();
		break;
	}
}

void ProgramManager::menu_AddLayer(int value)
{
	switch(value)
	{
	case 1:
		m_Manager->m_layerManager.AddLayer(RenderableFactory::CreateRenderable(9));
		break;
	}
}


void ProgramManager::mousebutton(int button, int state, int x, int y)
{
	GLUI_String message("");

	if(state != GLUT_DOWN)
		m_Manager->m_MouseState = MOUSE_INACTIVE;
	else
	{
		m_Manager->mousedown_x = x;
		m_Manager->mousedown_y = y;

		if(button == GLUT_RIGHT_BUTTON)
		{
			m_Manager->m_MouseState = MOUSE_ZOOMING;
			message = "Move mouse to change zoom level of the current layer";
		}
		else if(button == GLUT_LEFT_BUTTON)
		{

			if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
			{
				m_Manager->m_MouseState = MOUSE_CUTOFF;
				message = "Move mouse to adjust parameter of the current layer";
			}
			else if(glutGetModifiers() & GLUT_ACTIVE_CTRL)
			{
				m_Manager->m_MouseState = MOUSE_ZOOMING;
				message = "Move mouse to change zoom level of the current layer";
			}
			else
			{
				m_Manager->m_MouseState = MOUSE_PANNING;
				message = "Move mouse to pan the current layer";
			}
		}
	}

	m_Manager->m_logEdit->set_text(message);
}

void ProgramManager::mousemove(int x, int y)
{
	if(m_Manager->m_MouseState != MOUSE_INACTIVE)
	{
		int delta_x = m_Manager->mousedown_x - x;
		int delta_y = -(m_Manager->mousedown_y - y);
		m_Manager->mousedown_x = x;
		m_Manager->mousedown_y = y;

		switch(m_Manager->m_MouseState)
		{
		case MOUSE_PANNING:
			m_Manager->m_layerManager.MoveCoordinates(delta_x, delta_y);
			break;
		case MOUSE_ZOOMING:
			m_Manager->m_layerManager.IncrementZoom(delta_y);
			break;
		case MOUSE_CUTOFF:
			m_Manager->m_layerManager.IncrementParameter(((float)delta_y)/100);
			break;
		}
	}
}

void ProgramManager::tick(int value)
{
	if(m_Manager->m_layerManager.ShouldUpdateLayers())
	{
		m_Manager->m_layerManager.UpdateTextures();

		//must set main GLUT window
		glutSetWindow(m_Manager->m_window_id);
		glutPostRedisplay();
	}

	glutTimerFunc(10, ProgramManager::tick, 0);
}


void ProgramManager::display(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();									// Reset The View

	m_Manager->m_layerManager.Render();

	glutSwapBuffers();
}

void ProgramManager::reshape(int w, int h)
{
	int x,y,w2,h2;
	GLUI_Master.get_viewport_area(&x, &y, &w2, &h2);
	//m_Manager->m_layerManager.SetViewportSize(w2, h2);

	GLUI_Master.auto_set_viewport();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glEnable(GL_TEXTURE_2D);
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);

}

int ProgramManager::Run(int argc, char **argv)
{
	int w, h, x, y;
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT);

	m_window_id = glutCreateWindow("NoiseViewer v1.75");

	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping ( NEW )
	glEnable(GL_BLEND);
	glBlendFunc (GL_SRC_ALPHA, GL_DST_ALPHA);
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LESS);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

	//if you need to use these functions, don't go through GLUT, use this:
	//GLUI_Master.set_glutKeyboardFunc( myGlutKeyboard );
	//GLUI_Master.set_glutSpecialFunc( myGlutSpecial );

	GLUI_Master.set_glutMouseFunc( ProgramManager::mousebutton );
	GLUI_Master.set_glutReshapeFunc( ProgramManager::reshape );

	glutTimerFunc(10, ProgramManager::tick, 0);
	glutMotionFunc(ProgramManager::mousemove);
	glutDisplayFunc(ProgramManager::display);
	GLUI_Master.get_viewport_area(&x, &y, &w, &h);

	GLUI *glui = GLUI_Master.create_glui_subwindow(m_window_id, GLUI_SUBWINDOW_RIGHT);
	glui->set_main_gfx_window(m_window_id);

	GLUI_Master.get_viewport_area(&x, &y, &w, &h);

	//int submenu_add_layer = glutCreateMenu(ProgramManager::menu_AddLayer);
	//glutAddMenuEntry("Basic noise", 1);
	//int menu_noise = glutCreateMenu(ProgramManager::menu_Reset);
	//glutAddMenuEntry("Reset camera", 1);
	//glutAddSubMenu("Add layer", submenu_add_layer);

	//glutAttachMenu(GLUT_RIGHT_BUTTON);

	GLUI_Master.set_glutIdleFunc(NULL);
	GLUI *log_window = GLUI_Master.create_glui_subwindow(m_window_id, GLUI_SUBWINDOW_BOTTOM);
	log_window->set_main_gfx_window(m_window_id);
	m_logEdit = log_window->add_edittext("Info: ");
	m_logEdit->set_alignment(GLUI_ALIGN_LEFT);
	m_logEdit->set_w(400);

	m_layerManager.Initialize(glui);

	GLUI_Master.get_viewport_area(&x, &y, &w, &h);
	m_layerManager.SetViewportSize(w, h);

	m_layerManager.AddLayer(RenderableFactory::CreateRenderable(0));
	m_layerManager.UpdateTextures();

//	glui->set_main_gfx_window( m_window_id );

	glutMainLoop();
	return 0;
}