#include "RenderableFactory.h"

#include "NoiseSlice.h"
#include "SquareOverlay.h"
#include "Circles.h"

LayerTypeInfo RenderableFactory::TypeInformation[] = {
	{"Noise"},
	{"Squares"},
	{"Circles"}
};

int RenderableFactory::LayerTypeCount = sizeof(TypeInformation) / sizeof(LayerTypeInfo);


Renderable* RenderableFactory::CreateRenderable(int renderable_type)
{
	if(renderable_type < 0 || renderable_type >= LayerTypeCount)
		return NULL;

	Renderable *retval;

	switch(renderable_type)
	{
	case 0:
		retval = new NoiseSlice(renderable_type);
		break;
	case 1:
		retval = new SquareOverlay(renderable_type);
		break;
	case 2:
		retval = new Circles(renderable_type);
		break;
	default:
		return NULL;
		break;
	}

	retval->Initialize();
	return retval;
}

RenderableFactory::RenderableFactory(void)
{
	//private constructor, nobody interesting should call this.
}

RenderableFactory::~RenderableFactory(void)
{
}