#include "WindowMapping.h"
#include <stdio.h>

WindowMapping::WindowMapping()
{
	spaceWindow.bottom = spaceWindow.left = spaceWindow.right = spaceWindow.top = 0.0f;
	bitmapWindow.bottom = bitmapWindow.left = bitmapWindow.right = bitmapWindow.top = 0.0f;
	m_bitmapWidth = m_bitmapHeight = m_windowWidth = m_windowHeight = 0.0f;
}

WindowMapping::~WindowMapping()
{
}

void WindowMapping::SetCoordinates(float top, float right, float left, float bottom)
{
	assert(right > left);
	assert(bottom > top);

	spaceWindow.top = top;
	spaceWindow.bottom = bottom;
	spaceWindow.right = right;
	spaceWindow.left = left;

	m_windowWidth = right - left;
	m_windowHeight = top - bottom;
}

void WindowMapping::SetDestinationWindow(float top, float right, float left, float bottom)
{
	assert(right > left);
	assert(bottom > top);

	bitmapWindow.top = top;
	bitmapWindow.bottom = bottom;
	bitmapWindow.right = right;
	bitmapWindow.left = left;

	m_bitmapWidth = right - left;
	m_bitmapHeight = bottom - top;
}

void WindowMapping::PixelPositionToSpace(float & x, float & y)
{
	float fractionX = (x - bitmapWindow.left) / (m_bitmapWidth);
	float fractionY = (y - bitmapWindow.top) / (m_bitmapHeight);

	x = spaceWindow.left + m_windowWidth*fractionX;
	y = spaceWindow.top + m_windowHeight*fractionY;
}

void WindowMapping::GetPixelSizeInSpace(float& xpixelsize, float& ypixelsize)
{
	float x = 1.0, y = 1.0;
	float fractionX = x / ((float)m_bitmapWidth);
	float fractionY = y / ((float)m_bitmapHeight);

	xpixelsize = m_windowWidth*fractionX;
	ypixelsize = m_windowHeight*fractionY;
	//LOG(L"Pixel size in space: (%g, %g)", xpixelsize, ypixelsize);
}

void WindowMapping::SpaceToPixel(float xpos, float ypos, int &x, int &y)
{
	xpos -= spaceWindow.left;
	ypos -= spaceWindow.top;

	x = m_bitmapWidth * (xpos / m_windowWidth);
	y = m_bitmapHeight * (ypos / m_windowHeight);
	
	//reverse this:
	//float fractionX = (x - bitmapWindow.left) / (m_bitmapWidth);
	//float fractionY = (y - bitmapWindow.top) / (m_bitmapHeight);

	//x = spaceWindow.left + m_windowWidth*fractionX;
	//y = spaceWindow.top + m_windowHeight*fractionY;


}

void WindowMapping::PixelDeltaToDelta(float & xdiff, float & ydiff)
{
	float xpixel, ypixel;
	GetPixelSizeInSpace(xpixel, ypixel);
	xdiff = xdiff * xpixel;
	ydiff = ydiff * ypixel;
}